#include <vector>
#include <iostream>

void MultiReader()
{
  Int_t nVars = 9;
  std::vector<TString> varNames (nVars);

  varNames[0]="TMVACoreHC";
  varNames[1]="TMVAFirstChiSqRatio";
  varNames[2]="TMVANormTkr1Hits";
  varNames[3]="TMVANormVtxAngle";
  varNames[4]="TMVANumVertices";
  varNames[5]="TMVAVtxMaxDistCenter";
  varNames[6]="Tkr1ToTFirst";
  varNames[7]="Tkr1ToTTrAve";
  varNames[8]="VtxZDir";

  // Initialize input values of both readers to 0
  std::vector<Float_t> varVals1 (nVars,0);
  std::vector<Float_t> varVals2 (nVars,0);

  // Create first reader, add variables, and book method
  TMVA::Reader *reader1 = new TMVA::Reader("Silent");
  for (int i(0); i < nVars; i++) {
    reader1->AddVariable( varNames[i], &varVals1[i]);
  }
  reader1->BookMVA( "BDT", "TMVA_node_28_BDT.weights.xml");

  // Evaluate first reader with input values of 0
  std::cout<<"Evaluating reader1 with input values = "<<varVals1[0]<<std::endl
	   <<"reader1: "<<reader1->EvaluateMVA("BDT")<<std::endl<<std::endl;


  // Create second reader, add variables, and book method
  // This reader shouldn't be Silent, but somehow it is...
  TMVA::Reader *reader2 = new TMVA::Reader("");
  for (int i(0); i < nVars; i++) {
    reader2->AddVariable( varNames[i], &varVals2[i]);
  }
  reader2->BookMVA( "BDT", "TMVA_node_28_BDT.weights.xml");

  // Evaluate second reader with input values of 0
  std::cout<<"Evaluating reader2 with input values = "<<varVals2[0]<<std::endl
	   <<"reader2: "<<reader2->EvaluateMVA("BDT")<<std::endl<<std::endl;


  // Set second reader input values to 1
  for (int i(0); i < nVars; i++) { varVals2[i] = 1; }
  std::cout<<"Resetting input values of reader2 = "<<varVals2[0]<<std::endl
	   <<"Keeping input values of reader1 = "<<varVals1[0]<<std::endl<<std::endl;

  // This shouldn't effect the output of reader 1, but it does
  std::cout<<"Evaluating reader1 with input values = "<<varVals1[0]<<std::endl
	   <<"reader1: "<<reader1->EvaluateMVA("BDT")<<std::endl<<std::endl;


  // From a brief look through the code, it seems as though DataSetInfo/DataSet
  // objects are accessed by name.  However, when a Reader is created, a 
  // default name is given to the DataSetInfo they create.  Thus, reader1 
  // getting linked to the data of reader2.

  delete reader1;
  delete reader2;
}
